import logging
# Suppress logs below WARNING level
logging.basicConfig(level=logging.WARNING)
import struct
from spl.token.instructions import create_associated_token_account, get_associated_token_address, close_account, CloseAccountParams
from typing import Optional,Union
from solana.transaction import AccountMeta, Transaction
from solders.pubkey import Pubkey
from solders.instruction import Instruction
from solders.compute_budget import set_compute_unit_limit, set_compute_unit_price
from .account_utils import payer_keypair,get_payer_keypair
from .isolated_rpcs import *
from abstract_solcatcher import abstract_solana_rate_limited_call
from solana.rpc.types import TxOpts
from solana.rpc.types import TokenAccountOpts
from .utils import *
from .account_utils import *
seeds = [b"event_authority"]
GLOBAL = get_pubKey("4wTV1YmiEkRvAtNtsSGPtUrqRYQMe5SKy2uB4Jjaxnjf")
FEE_RECIPIENT = get_pubKey("CebN5WGQ4jvEPvsVU4EoHEpgzq1VV7AbicfhtW4xC9iM")
TOKEN_PROGRAM = get_pubKey("TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA")
ASSOC_TOKEN_ACC_PROG = get_pubKey("ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL")
RENT = get_pubKey("SysvarRent111111111111111111111111111111111")
EVENT_AUTHORITY = get_pubKey("Ce6TQqeHC9p8KetsN6JsjHK7UTZk7nasjjnr7XxXp9F1")
GLOBAL = get_pubKey("4wTV1YmiEkRvAtNtsSGPtUrqRYQMe5SKy2uB4Jjaxnjf")
SYSTEM_PROGRAM = get_pubKey("11111111111111111111111111111111")
PUMP_FUN_PROGRAM_ID = get_pubKey("6EF8rrecthR5Dkzon8Nwu78hRvfCKubJ14M5uBEwF6P")
event_authority_pubkey, bump = Pubkey.find_program_address(seeds, PUMP_FUN_PROGRAM_ID)
LAMPORTS_PER_SOL = 1_000_000_000
UNIT_PRICE =  1_000_000
UNIT_BUDGET =  100_000



from solders.pubkey import Pubkey
def get_coin_data(mint_str):
    return get_pump_fun_data(str(mint_str))      
def get_virtual_reserves(bonding_curve: Pubkey):
    bonding_curve_struct = Struct(
        Padding(8),
        "virtualTokenReserves" / Int64ul,
        "virtualSolReserves" / Int64ul,
        "realTokenReserves" / Int64ul,
        "realSolReserves" / Int64ul,
        "tokenTotalSupply" / Int64ul,
        "complete" / Flag
    )
    
    try:
        account_info = client.get_account_info(bonding_curve)
        data = account_info['value']['data']
        parsed_data = bonding_curve_struct.parse(data)
        return parsed_data
    except Exception:
        return None
def derive_bonding_curve_accounts(mint_str: str):
    try:
        mint = Pubkey.from_string(mint_str)
        bonding_curve, _ = Pubkey.find_program_address(
            ["bonding-curve".encode(), bytes(mint)],
            PUMP_FUN_PROGRAM
        )
        associated_bonding_curve = get_associated_token_address(bonding_curve, mint)
        return bonding_curve, associated_bonding_curve
    except Exception:
        return None, None
def getKeys(coin_data,token_account,owner,buy=True):
        coin_data = get_pump_fun_data(str(mint_str))
        MINT = Pubkey.from_string(coin_data['mint'])
        BONDING_CURVE = Pubkey.from_string(coin_data['bonding_curve'])
        ASSOCIATED_BONDING_CURVE = Pubkey.from_string(coin_data['associated_bonding_curve'])
        ASSOCIATED_USER = Pubkey.from_string(str(token_account))
        USER = Pubkey.from_string(str(payer_keypair.pubkey()))
        PUMP_FUN_TOKEN_PROGRAM_SWITCH = PUMP_FUN_TOKEN_PROGRAM if buy else PUMP_FUN_ASSOC_TOKEN_ACC_PROG
        PUMP_FUN_RENT_PROGRAM_SWITCH = PUMP_FUN_RENT if buy else PUMP_FUN_TOKEN_PROGRAM
        # Build account key list 
        keys = [
            AccountMeta(pubkey=PUMP_FUN_GLOBAL, is_signer=False, is_writable=False),
            AccountMeta(pubkey=PUMP_FUN_FEE_RECIPIENT, is_signer=False, is_writable=True),
            AccountMeta(pubkey=MINT, is_signer=False, is_writable=False),
            AccountMeta(pubkey=BONDING_CURVE, is_signer=False, is_writable=True),
            AccountMeta(pubkey=ASSOCIATED_BONDING_CURVE, is_signer=False, is_writable=True),
            AccountMeta(pubkey=ASSOCIATED_USER, is_signer=False, is_writable=True),
            AccountMeta(pubkey=USER, is_signer=True, is_writable=True),
            AccountMeta(pubkey=PUMP_FUN_SYSTEM_PROGRAM, is_signer=False, is_writable=False), 
            AccountMeta(pubkey=PUMP_FUN_TOKEN_PROGRAM_SWITCH, is_signer=False, is_writable=False),
            AccountMeta(pubkey=PUMP_FUN_RENT_PROGRAM_SWITCH, is_signer=False, is_writable=False),
            AccountMeta(pubkey=PUMP_FUN_EVENT_AUTHORITY, is_signer=False, is_writable=False),
            AccountMeta(pubkey=PUMP_FUN_PROGRAM, is_signer=False, is_writable=False)
        ]
        return keys
